function [C] = colors(spec,interval,st,lso,lsource)
% function [C] = colors(spec,interval,st,lso)
% function [C] = colors(spec,interval,st,lso,'lsource')
%
% Calculates color co-ordinates for a spectrum.
%
% Only the values measured from the wavelength range [400 nm, 700 nm]
% are used in calculation. The spectrum should be sampled at 5 nm or
% 10 nm intervals. If some other sampling interval is used then
% cubic spline interpolation is used to get the intensity values at
% the wavelengths from 400 nm to 700 nm at 5 nm sampling intervals.
% 
%
% INPUT:
%
%	spec 		spectrum 2*n matrix, the 1st column is wavelength
%			and the 2nd column is intensity (spectrum)
%	interval 	interval between samples
%	st		matrix of tristimulus curves (file st380_780_5.dat)
%	lso		matrix of light sources (file ls300_830_5.dat
%
%	By default colors uses D65 light source.
%
%	[C] = colors(spec,interval,st,lso,'lsource') specifies alternate
%	light source types. Available source types are
%
%		'A'
%		'B'
%		'C'
%		'D65'
%	
% OUTPUT:
%
% 	C(1) 	x	Chromaticity co-ordinates
% 	C(2) 	y	Chromaticity co-ordinates
% 	C(3) 	z	Chromaticity co-ordinates
% 	C(4) 	X	Tristimulus value
% 	C(5) 	Y	Tristimulus value
% 	C(6) 	Z	Tristimulus value
% 	C(7)	R	Tristimulus value
% 	C(8) 	G	Tristimulus value
% 	C(9) 	B	Tristimulus value
% 	C(10) 	L*	CIE 1976 lightness
% 	C(11) 	a*	CIELAB
% 	C(12)	b*	CIELAB
% 	C(13) 	u'	CIE 1976 UCS diagram
% 	C(14) 	v'	CIE 1976 UCS diagram
% 	C(15) 	u*	CIELUV
% 	C(16) 	v*	CIELUV

%************************************************************************
%  (C) J. Haanpalo      1994-1995                                       *
%                                                                       *
%  NOTE: This program is copyrighted in the sense that it               *
%  may be used for scientific purposes. The program as a whole, or      *
%  parts thereof, cannot be included or used in any commercial          *
%  application without written permission granted by its producents.    *
%                                                                       *
%  All comments concerning this program may be sent to the              *
%  e-mail address 'haanpalo@lut.fi'.                                    *
%                                                                       *
%***********************************************************************/

ciex=[];
ciey=[];
ciez=[];
[i,j]=size(spec);

% If the interval between samples is 5nm or 10nm
if (interval == 5 | interval == 10)
	w = spec(1,1):interval:spec(i,1);
	step = interval/5;
	ciex=st(5:step:step*i+4,2);
	ciey=st(5:step:step*i+4,3);
	ciez=st(5:step:step*i+4,4);
else
% We have to interpolate the missing points of the spectrum
	w = 400:5:700;
	newspec = interp1(spec(:,1), spec(:,2), w, 'spline');
	spec = [w' newspec];
	[i,j]=size(spec);

	ciex=st(5:65,2);
	ciey=st(5:65,3);
	ciez=st(5:65,4);
end

% Select light source
% Default is D65 (5th column in lso matrix)
lcol = 5;
if(nargin > 4)
	if(strcmp(lsource, 'A'))
		lcol = 2;
	end
	if(strcmp(lsource, 'B'))
		lcol = 3;
	end
	if(strcmp(lsource, 'C'))
		lcol = 4;
	end
end
light = lso(21:81,lcol);

% 400nm is 101st element in lso matrix (old stuff)
%light = lso(w - 299,3);

white = ones(i, 1);

k = 100.0 / sum(white .* ciey .* light);
X0 = k * sum(white .* ciex .* light);
Y0 = k * sum(white .* ciey .* light);
Z0 = k * sum(white .* ciez .* light);

%%%%%%%%%% CIE XYZ co-ordinates %%%%%%%%%%%%%%%%%%%%%%%%%%
X = k * sum(spec(:,2) .* ciex .* light);
Y = k * sum(spec(:,2) .* ciey .* light);
Z = k * sum(spec(:,2) .* ciez .* light);

%%%%%%%%%% Relative XYZ co-ordinates %%%%%%%%%%%%%%%%%%%%%
div = X + Y + Z;
x = X/div;
y = Y/div;
z = Z/div;

%%%%%%%%%% RGB co-ordinates %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
R = 2.3646*X - 0.8965*Y - 0.4681*Z;
G = -0.5152*X + 1.4264*Y + 0.0888*Z;
B = 0.0052*X - 0.0144*Y + 1.0092*Z;

%%%%%%%%%% CIELAB co-ordinates %%%%%%%%%%%%%%%%%%%%%%%%%%%
FX = X/X0;
FY = Y/Y0;
FZ = Z/Z0;
if (FY > 0.008856)
	Lt = 116.0 * FY^(1.0/3.0) - 16.0;
	YY = FY;
else
	Lt = 903.3 * FY;
	YY = 7.787 * FY + 16/116;
end
if (FX > 0.008856)
	XX = FX;
else
	XX = 7.787 * FX + 16/116;
end
if (FZ > 0.008856)
	ZZ = FZ;
else
	ZZ = 7.787 * FZ + 16/116;
end
at = 500 * (XX^(1.0/3.0) - YY^(1.0/3.0));
bt = 200 * (YY^(1.0/3.0) - ZZ^(1.0/3.0));

%%%%%%%%%% CIELUV co-ordinates %%%%%%%%%%%%%%%%%%%%%%%%%%%
u = 4*x / (-2*x + 12*y + 3);
v = 6*y / (-2*x + 12*y + 3);
up = 4*X / (X + 15*Y + 3*Z);
vp = 9*Y / (X + 15*Y + 3*Z);
up0 = 4*X0 / (X0 + 15*Y0 + 3*Z0);
vp0 = 9*Y0 / (X0 + 15*Y0 + 3*Z0);
ut = 13 * Lt * (up - up0);
vt = 13 * Lt * (vp - vp0);

%%%%%%%%%% Construct output vector %%%%%%%%%%%%%%%%%%%%%%%
C = [x y z X Y Z R G B Lt at bt u v ut vt]; 
